

#include "HSolidModel.h"
#include "HTools.h"

#include "hc.h"

#include <string.h>
#include <stdio.h>
#include <ctype.h>

#ifdef PARASOLID
#include "parasolid_kernel.h"
#include "kernel_interface.h"
#include "hp_bridge.h"
#endif

#ifdef ACIS
#include "ha_bridge.h"
#include "kernel/kernapi/api/kernapi.hxx" 
#endif


HSolidModel::HSolidModel(HDB* hDB) : HBaseModel(hDB)
{
#ifdef PARASOLID

	PK_PARTITION_create_empty(&m_partition);

#endif

	m_solid_model = true;
}

HSolidModel::~HSolidModel()
{
	// Model cleanup : delete all the entities from the 
	// partition associated with this model
#ifdef PARASOLID
	int n_bodies;
	PK_BODY_t	*bodies;

	PK_PARTITION_ask_bodies(m_partition, &n_bodies, &bodies);
	PK_ENTITY_delete(n_bodies, bodies);
	
	if (n_bodies > 0)
		PK_MEMORY_free(bodies);
#endif
#ifdef ACIS
	ENTITY* e;
	m_entityList.init();
	e = m_entityList.next();
	while (e != NULL)
	{
		DeleteAcisEntity(e);
		e = m_entityList.next();
	}
	m_entityList.clear();
#endif

}

#ifdef ACIS
void HSolidModel::DeleteAcisEntity( ENTITY* entity)
{
	ENTITY_LIST elist;
	elist.add(entity);
	HA_Delete_Entity_Geometry(elist);
	// delete the ACIS body
	api_del_entity(entity);
}
void HSolidModel::AddAcisEntity( ENTITY* entity)
{
	m_entityList.add(entity);
}
void HSolidModel::RemoveAcisEntity( ENTITY* entity)
{
	m_entityList.remove(entity);
}
#endif
#ifdef PARASOLID
void HSolidModel::DeleteParasolidEntity( PK_ENTITY_t entity)
{
	HP_Delete_Entity_Geometry(1, &entity);
	PK_ENTITY_delete(1, &entity);
}
#endif

// our application-specific read function
bool HSolidModel::Read(const char * FileName) 
{   
	unsigned int i;
	char extension[120]; 
	bool success = true;

	extension[0]='\0';
	HC_Parse_String(FileName,".",-1,extension);

	/*make it lower case*/
	for(i=0; i<strlen(extension);i++)
		extension[i]=(char) tolower(extension[i]);

	// read the file into the model object's model segment
    HC_Open_Segment_By_Key(m_ModelKey);	

	// read each file into it's own subsegment of the main model segment 
	// so that we can easily apply transformations to different parts that might be read into 
	// the same model object; currently not supported in MVO, but could be added in the future
	// This is also required so that when we do segment level deletion later on, we don't 
	// accidentally delete the model_key segment!

		HC_Open_Segment("");

#ifdef PARASOLID
			if  (streq(extension,"xmt_txt") || streq(extension,"x_t")) 
			{
				PK_PART_receive_o_t options;
				PK_ERROR_code_t		result;

				PK_PART_receive_o_m(options);

				options.transmit_format = PK_transmit_format_text_c;
				result = HP_Read_Xmt_File(FileName, GetPartition(), 0, 0, &options);	// read an XMT file
				
				if (result < 0) success = false;
			} 
			else
#endif
#ifdef ACIS
			if  (streq(extension,"sat"))
			{  
				ENTITY_LIST elist;
				ENTITY* entity;

				HA_Read_Sat_File(FileName, elist);	// read an SAT file
				elist.init();						// set next() function to return first element in "list"

				while (entity = elist.next())
				{
					m_entityList.add(entity);
				}
			} 
			else
#endif
				success = false;
		HC_Close_Segment();
    HC_Close_Segment();

    return success;
}

// our application-specific write function
bool HSolidModel::Write(const char * FileName, HBaseView * view, int width, int height) 
{   
	unsigned int i;
	char extension[120]; 
	bool success = true;

	extension[0]='\0';
	HC_Parse_String(FileName,".",-1,extension);

	//make it lower case
	for(i=0; i<strlen(extension);i++)
		extension[i]=(char) tolower(extension[i]);


#ifdef PARASOLID
		if  (streq(extension,"xmt_txt") || streq(extension,"x_t")) 
		{
			int						n_bodies;
			PK_BODY_t				*bodies;
			PK_PART_transmit_o_t	options;
			PK_ERROR_code_t			result;

			PK_PART_transmit_o_m(options);

			options.transmit_format = PK_transmit_format_text_c;

			PK_PARTITION_ask_bodies(m_partition, &n_bodies, &bodies);
			result = HP_Write_Xmt_File(FileName, n_bodies, bodies, &options);	// write an XMT file

			if (result < 0) success = false;

			if (n_bodies > 0)
				PK_MEMORY_free(bodies);
		}
		else
#endif

#ifdef ACIS
		if (streq(extension,"sat"))
		{
			if (m_entityList.count() > 0)
			HA_Write_Sat_File(FileName, m_entityList);
		}
		else
#endif
		{
			if(!HBaseModel::Write(FileName, view, width, height))
				success = false;
		}
 
    return success;
}





